﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;

namespace Ld31
{
    public class AudioEngine : Root
    {
        public AudioClip SelectSound;
        public AudioClip BuildSound;
        public AudioClip RemoveSound;
        public AudioClip BlockedSound;
        public AudioClip TextSound;
        public AudioClip PowerUpSound;

        List<AudioSource> Sources = new List<AudioSource>();

        public void PlaySound(Vector3 position, AudioClip clip, float volume)
        {
            GameObject go = new GameObject("SoundSource");
            go.transform.position = position;
            var source = go.AddComponent<AudioSource>();
            source.loop = false;
            source.clip = clip;
            source.volume = volume;
            Sources.Add(source);
            source.Play();
        }

        private void PlayBuildSound(object caller, EventArgs e)
        {
            PlaySound(Game.Ui.Zeiger.transform.position, BuildSound, .1f);
        }

        private void PlaySelectSound(object caller, EventArgs e)
        {
            PlaySound(Game.Ui.Zeiger.transform.position, SelectSound, .1f);
        }

        private void PlayRemoveSound(object caller, EventArgs e)
        {
            PlaySound(Game.Ui.Zeiger.transform.position, RemoveSound, .1f);
        }

        private void PlayBlockedSound(object caller, EventArgs e)
        {
            PlaySound(Game.Ui.Zeiger.transform.position, BlockedSound, .1f);
        }

        private void PlayPowerUpSound(object caller, EventArgs e)
        {
            PlaySound(new Vector3(90, 50, 0), PowerUpSound, .1f);
        }

        private void PlayTextSound(object caller, EventArgs e)
        {
            PlaySound(new Vector3(90, 50, 0), TextSound, .005f);
        }

        void OnEnable()
        {
            MapGenerator.OnBuild += PlayBuildSound;
            Interface.OnBlocked += PlayBlockedSound;
            MapGenerator.OnRemove += PlayRemoveSound;
            Toolbar.OnSelect += PlaySelectSound;
            TitleScreen.OnText += PlayTextSound;
            Structure.OnCollect += PlayPowerUpSound;
        }

        void OnDisable()
        {
            MapGenerator.OnBuild -= PlayBuildSound;
            Interface.OnBlocked -= PlayBlockedSound;
            MapGenerator.OnRemove -= PlayRemoveSound;
            Toolbar.OnSelect -= PlaySelectSound;
            TitleScreen.OnText -= PlayTextSound;
            Structure.OnCollect -= PlayPowerUpSound;

        }

        // Use this for initialization
        void Start()
        {
	
        }
	
        // Update is called once per frame
        void Update()
        {
            for (int i = 0; i < Sources.Count; i++)
            {
                if (!Sources[i].isPlaying)
                {
                    Destroy(Sources[i].gameObject);
                    Sources.RemoveAt(i);
                }
            }
        }
    }
}