﻿using UnityEngine;
using System.Collections;
using System;

namespace Ld31
{
    [RequireComponent(typeof(MeshFilter))]
    [RequireComponent(typeof(MeshRenderer))]
    public class Fog : Root
    {
        private Texture2D _texture;
        MeshRenderer renderer;

        // Use this for initialization
        void Start()
        {
            BuildMesh(Game.Config.Width, Game.Config.Height);
            BuildTexture();
            //UpdateTextureWorker(null, EventArgs.Empty);
            renderer = GetComponent<MeshRenderer>();
        }

        void OnEnable()
        {
            MapGenerator.OnBuild += UpdateTextureWorker;
            MapGenerator.OnRemove += UpdateTextureWorker;
        }

        void OnDisable()
        {
            MapGenerator.OnBuild -= UpdateTextureWorker;
            MapGenerator.OnRemove -= UpdateTextureWorker;
        }
        // Update is called once per frame
        void Update()
        {
            if (Game.Ui.Toolbar.Selected != null && Game.Ui.Toolbar.Selected.Type == ToolType.BUILDING)
                renderer.enabled = true;
            else
                renderer.enabled = false;
        }

        /// <summary>
        /// This visualizes the Data in map.Data.Explored.
        /// </summary>
        public void UpdateTextureWorker(object caller, EventArgs e)
        {
            //Misc.Log(xMin, xMax, yMin, yMax);
            var yStart = 0;
            var yStop = Game.Config.Height;
            var xStart = 0;
            var xStop = Game.Config.Width;
            byte alpha = 255;
            var colors = new Color32[Game.Config.Height * Game.Config.Width];

            for (int y = yStart; y < yStop; y++)
            {
                for (int x = xStart; x < xStop; x++)
                {
                    alpha = 255;
                    if (Game.Map._Data[x, y].InRange)
                    {
                        alpha = 0;
                    }
                    if (y % 255 == 0)
                    {
                    }
                    colors[y * Game.Config.Width + x] = new Color32(220, 40, 0, alpha);
                }
            }
            _texture.SetPixels32(colors);
            _texture.Apply();
        }

        public void BuildTexture()
        {
            int texWidth = Game.Config.Width;
            int texHeight = Game.Config.Height;
            _texture = new Texture2D(texWidth, texHeight);
            _texture.filterMode = FilterMode.Point;

            var colors = new Color32[Game.Config.Height * Game.Config.Width];
            for (int i = 0; i < colors.Length; i++)
            {
                colors[i] = new Color32(0, 0, 0, 255);
            }
            _texture.SetPixels32(colors);
            _texture.Apply();

            MeshRenderer mesh_renderer = GetComponent<MeshRenderer>();
            mesh_renderer.sharedMaterials[0].mainTexture = _texture;

            Debug.Log("FogOfWar Done");
        }

        private void BuildMesh(int width, int height)
        {

            int numVerts = width * height;

            // Generate Mesh Data
            Vector3[] vertices =
                {
                    new Vector3(0, 0, 0),
                    new Vector3(width, 0, 0),
                    new Vector3(0, height, 0),
                    new Vector3(width, height, 0)
                };
            Vector3[] normals =
                {
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1)
                };
            Vector2[] uv =
                {
                    new Vector2(0, 0),
                    new Vector2(1, 0),
                    new Vector2(0, 1),
                    new Vector2(1, 1)
                };
            int[] triangles =
                {
                    0, 1, 2,
                    2, 3, 1
                };

            // Create new Mesh and populate it with data
            Mesh mesh = new Mesh();
            mesh.vertices = vertices;
            mesh.triangles = triangles;
            mesh.normals = normals;
            mesh.uv = uv;

            // Assign our Mesh to our filter/collider/renderer
            MeshFilter mesh_filter = GetComponent<MeshFilter>();
            MeshCollider mesh_collider = GetComponent<MeshCollider>();

            mesh_filter.mesh = mesh;
            mesh_collider.sharedMesh = mesh;

            Debug.Log("Done Mesh");
        }
    }
}