﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;
using System.Linq;

namespace Ld31
{
    public enum SoilType
    {
        Any = 0,
        Grass = 1,
        Water = 2,
        Dirt = 3,
        Rubble = 4,
        Pepples = 5,
        Shrubbery = 6,
        Stubble = 7,
        Iron = 8,
        Stone = 9,
        Wood = 10,
        Rocks = 11,
        Ferns = 12,
        Lichen = 13,
        Moss = 14,
        Dust = 15
    }

    public class MapData
    {
        public Structure Structure = null;
        public int Soil = 0;
        public int MineralValue = 0;
        public int AgriculturalValue = 0;
        public bool InRange = false;
    }

    //[ExecuteInEditMode]
    public class MapGenerator : Root
    {
        public static event EventHandler OnBuild;
        public static event EventHandler OnBlocked;
        public static event EventHandler OnRemove;

        public MapData[,] _Data;
        private Texture2D _texture;
        private Color[][] _groundTiles;
        public Texture2D _GroundTilesAtlas;
        public List<Structure> _AllStructures = new List<Structure>();

        public GameObject PowerUpRuinsTemplate;
        public GameObject PowerUpCacheTemplate;
        public GameObject PowerUpXTemplate;

        public float[] rareDistribution = new float[8];
        public float[] commonDistribution = new float[8];

        // Use this for initialization
        void Start()
        {
            transform.position = new Vector3(0, 0, 0);

            _groundTiles = ChopUpTiles(_GroundTilesAtlas);
            BuildMesh(Game.Config.Width, Game.Config.Height);

            _Data = BuildData(Game.Config.Width, Game.Config.Height);

            BuildTexture();

            SpawnPowerUps();
        }
	
        // Update is called once per frame
        void Update()
        {
	
        }

        public MapData[,] BuildData(int width, int height)
        {
            MapData[,] data = new MapData[width, height];
            FastSample rare = new FastSample(rareDistribution);
            FastSample common = new FastSample(commonDistribution);

            var offset = UnityEngine.Random.Range(0, 20000);
            for (int x = 0; x < Game.Config.Width; x++)
            {
                for (int y = 0; y < Game.Config.Height; y++)
                {
                    data[x, y] = new MapData();
                    float sample;
                    float sampleS;
                    float sampleM;
                    float sampleL;
                    float sampleX;

                    // soil
                    sampleL = ( 
                        SimplexNoise.Noise.Generate(
                            (float)x * .010f + offset,
                            (float)y * .010f) + 1f) * 1.5f;
                    sampleM = ( 
                        SimplexNoise.Noise.Generate(
                            (float)x * .07f + offset,
                            (float)y * 0.07f) + 1f) * 6f;
                    sampleS = ( 
                        SimplexNoise.Noise.Generate(
                            (float)x * .011f + offset,
                            (float)y * 0.011f) + 1f) * 10f;
                    sample = sampleL * (sampleM + sampleS);

                    // Split map into zones and spawn samples
                    if (sample > 10f)
                    {
                        data[x, y].Soil = rare.Next() + 8;
                    }
                    else if (sample < 5f && sampleM < 2f)
                    {
                        data[x, y].Soil = 2;
                    }
                    else if (sample < 6f && sampleS > 3f && sampleS < 5f)
                    {
                        data[x, y].Soil = 3;
                    }
                    else if (sample > 5f && sampleM < 3f)
                    {
                        data[x, y].Soil = 1;
                    }
                    else
                    {
                        data[x, y].Soil = common.Next();
                    }
                    if (sample < 3f && sampleS > 3f && sampleS < 5f) {
                        data[x, y].Soil = rare.Next() + 5;
                    }
                    if (data[x, y].Soil == (int)SoilType.Iron)
                        data[x, y].MineralValue = RandFuncs.RndRange(600, 2400);
                    if (data[x, y].Soil == (int)SoilType.Wood)
                        data[x, y].MineralValue = RandFuncs.RndRange(50, 200);
                    if (data[x, y].Soil == (int)SoilType.Stone)
                        data[x, y].MineralValue = RandFuncs.RndRange(200, 800);
                }
            }
            return data;
        }

        Color[][] ChopUpTiles(Texture2D atlas)
        {
            int numTilesPerRow = atlas.width / Game.Config.Resolution;
            int numRows = atlas.height / Game.Config.Resolution;
            Color[][] tiles = new Color[numTilesPerRow * numRows][];

            for (int y = 0; y < numRows; y++)
            {
                for (int x = 0; x < numTilesPerRow; x++)
                {
                    tiles[y * numTilesPerRow + x] = atlas.GetPixels(x * Game.Config.Resolution, y * Game.Config.Resolution, Game.Config.Resolution, Game.Config.Resolution);
                }
            }

            return tiles;
        }

        public bool IsColliding(int x, int y)
        {
            if (IsOnMap(x, y))
                return _Data[x, y].Structure != null;
            return false;
        }

        public void BuildTexture()
        {

            /* This builds a texture based on the Data in "mapData"
         * 
         * 
         */

            int texWidth = Game.Config.Resolution * Game.Config.Width;
            int texHeight = Game.Config.Resolution * Game.Config.Height;
            _texture = new Texture2D(texWidth, texHeight);

            for (int y = 0; y < Game.Config.Height; y++)
            {
                for (int x = 0; x < Game.Config.Width; x++)
                {
                    // default tile
                    Color[] p = _groundTiles[2];

                    int soil = Game.Map._Data[x, y].Soil;
                    if (soil > 0)
                        p = _groundTiles[soil];

                    _texture.SetPixels(x * Game.Config.Resolution, y * Game.Config.Resolution, Game.Config.Resolution, Game.Config.Resolution, p);
                }
            }

            _texture.filterMode = FilterMode.Point;
            _texture.Apply();

            MeshRenderer mesh_renderer = GetComponent<MeshRenderer>();
            mesh_renderer.sharedMaterials[0].mainTexture = _texture;

            Debug.Log("Done Texture");
        }

        private void BuildMesh(int width, int height)
        {

            int numVerts = width * height;

            // Generate Mesh Data
            Vector3[] vertices =
                {
                    new Vector3(0, 0, 0),
                    new Vector3(width, 0, 0),
                    new Vector3(0, height, 0),
                    new Vector3(width, height, 0)
                };
            Vector3[] normals =
                {
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1),
                    new Vector3(0, 0, 1)
                };
            Vector2[] uv =
                {
                    new Vector2(0, 0),
                    new Vector2(1, 0),
                    new Vector2(0, 1),
                    new Vector2(1, 1)
                };
            int[] triangles =
                {
                    0, 1, 2,
                    2, 3, 1
                };

            // Create new Mesh and populate it with data
            Mesh mesh = new Mesh();
            mesh.vertices = vertices;
            mesh.triangles = triangles;
            mesh.normals = normals;
            mesh.uv = uv;

            // Assign our Mesh to our filter/collider/renderer
            MeshFilter mesh_filter = GetComponent<MeshFilter>();
            MeshCollider mesh_collider = GetComponent<MeshCollider>();

            mesh_filter.mesh = mesh;
            mesh_collider.sharedMesh = mesh;

            Debug.Log("Done Mesh");
        }

        public bool IsOnMap(int x, int y)
        {
            return (x > 0 && x < Game.Config.Width && y > 0 && y < Game.Config.Height);
        }

        public void RemoveStructure(int posX, int posY)
        {
            if (IsColliding(posX, posY))
            {
                if (OnRemove != null)
                    OnRemove(_Data[posX, posY].Structure, EventArgs.Empty);
                _AllStructures.Remove(_Data[posX, posY].Structure);
                Instantiate(Game.Config.RemoveFX, _Data[posX, posY].Structure.gameObject.transform.position + new Vector3(0, 0, -2f), Quaternion.identity);
                Destroy(_Data[posX, posY].Structure.gameObject);
                _Data[posX, posY].Structure = null;
                RecalculateRangeMap();
                Debug.Log("Remove " + posX + ", " + posY);
            }
        }

        void MarkInRange(int posX, int posY, int range)
        {
            for (
                int y = Mathf.Clamp(posY - range, 0, Game.Config.Height);
                y < Mathf.Clamp(posY + range + 1, 0, Game.Config.Height);
                y++)
            {
                for (
                    int x = Mathf.Clamp(posX - range, 0, Game.Config.Width);
                    x < Mathf.Clamp(posX + range + 1, 0, Game.Config.Width);
                    x++)
                {
//                    Debug.Log(x + ", " +y);
                    _Data[x, y].InRange = true;
                }
            }
        }

        void RecalculateRangeMap()
        {
            foreach (var each in _AllStructures)
            {
                if (each.Type != StructureType.PowerUp)
                    MarkInRange(each.x, each.y, each.Range);
            }
        }

        /// <summary>
        /// Clone object and place it at x,y. Add references to all dataSets.
        /// </summary>
        public Structure AddStructure(
            int posX, 
            int posY, 
            GameObject prefab,
            bool silent = false
        )
        {
            var templateLocation = prefab.GetComponent<Structure>();

            if (templateLocation == null)
                throw new Exception("AddLocation: Template/Prefab must have a 'Structure' Component." + prefab.name);


            if (IsOnMap(posX, posY))
            {
                // only build if the space isn't occupied
                if (IsColliding(posX, posY))
                {
                    Debug.Log("AddLocation: Cannot add " + templateLocation.Name + " at " + posX + ", " + posY);
                    return null;
                }
                else
                {
                    // instatiate
                    GameObject newGo = (GameObject)Instantiate(prefab);
                    Structure newStruct = newGo.GetComponent<Structure>();
                    if (newStruct == null)
                        Debug.Log("Null " + prefab);

                    // position
                    newGo.transform.position = new Vector3(posX, posY, -1f);
                    if (!silent)
                    {
                        if (!Game.Ui.firstBuildingPlaced)
                            Game.Ui.firstBuildingPlaced = true;
                        MarkInRange(posX, posY, newStruct.Range);
                        Instantiate(Game.Config.BuildFX, newGo.transform.position + new Vector3(0, 0, -1f), Quaternion.identity);
                    }

                    // references
                    _AllStructures.Add(newStruct);
                    _Data[posX, posY].Structure = newStruct;

                    Debug.Log("Location " + newStruct.Name + " spawned at " + newStruct.x + ", " + newStruct.y + " / " + newStruct.GetInstanceID());
                    if (!silent)
                    {
                        if (OnBuild != null)
                            OnBuild(newStruct, EventArgs.Empty);
                    }

                    return newStruct;
                }
            }
            Debug.Log("AddLocation: Out of bounds.");
            return null;
        }

        void SpawnPowerUps()
        {

            for (int i = 0; i < 9; i++)
            {
                var x = UnityEngine.Random.Range(0, Game.Config.Width);
                var y = UnityEngine.Random.Range(0, Game.Config.Height);
                if (!(new []{ 1, 2, 3, 8, 9, 10 }.Contains(_Data[x, y].Soil)))
                {
                    var powerUp = AddStructure(x, y, PowerUpRuinsTemplate, true);
                }
            }


            for (int i = 0; i < 9; i++)
            {
                var x = UnityEngine.Random.Range(0, Game.Config.Width);
                var y = UnityEngine.Random.Range(0, Game.Config.Height);
                if (!(new []{ 1, 2, 3, 8, 9, 10 }.Contains(_Data[x, y].Soil)))
                {
                    var powerUp = AddStructure(x, y, PowerUpCacheTemplate, true);
                }
            }


            for (int i = 0; i < 9; i++)
            {
                var x = UnityEngine.Random.Range(0, Game.Config.Width);
                var y = UnityEngine.Random.Range(0, Game.Config.Height);
                if (!(new []{ 1, 2, 3, 8, 9, 10 }.Contains(_Data[x, y].Soil)))
                {
                    var powerUp = AddStructure(x, y, PowerUpXTemplate, true);
                }
            }

        }
    }
}