using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System;

namespace Ld31
{
    public enum StructureType
    {
        Mine,
        Farm,
        Workshop,
        Market,
        House,
        PowerUp
    }

    public class Structure : Root
    {
        static public event EventHandler OnRemove;
        static public event EventHandler OnCollect;

        public StructureType Type;
        public String Name;
        public int Range = 2;

        public Inventory BuildRessources;
        public Inventory ConsumedRessources;
        public Inventory ProducedRessources;

        public SoilType SoilType = SoilType.Any;

        public float Interval = 1f;
        public bool IsActive = false;
        public bool IsExhausted = false;

        SpriteRenderer spriteRenderer;

        void Start()
        {
            foreach (var each in BuildRessources.Items)
            {
                Game.Economy.Inventory[each.Key] -= each.Value;
            }

            if (Type != StructureType.PowerUp)
            {
                Game.Clock.XInvokeRepeating(this, "Produce", Interval, Interval);
                Game.Clock.XInvokeRepeating(this, "Consume", Interval, Interval);
                Game.Clock.XInvokeRepeating(this, "Inhabitants", Interval, Interval);
                Game.Clock.XInvokeRepeating(this, "ActiveFX", 1f, 10f);
            }
            else
            {
                IsActive = true;
            }

            spriteRenderer = GetComponent<SpriteRenderer>();
        }

        void ActiveFX()
        {
            if (IsActive)
                Instantiate(Game.Config.ActiveFX, gameObject.transform.position + new Vector3(0, 0, -2f), Quaternion.identity);
        }

        void Update()
        {
            if (IsActive)
                spriteRenderer.color = new Color32(255, 255, 255, 255);
            else
                spriteRenderer.color = new Color32(125, 125, 125, 255);

        }

        void Inhabitants()
        {
            if (Type == StructureType.House)
            {
            }
        }

        void Produce()
        {
            if (IsActive && !IsExhausted)
            {
                foreach (var each in ProducedRessources.Items)
                {
                    Game.Economy.Inventory[each.Key] += each.Value;
                }
            }
        }

        void Consume()
        {
            if (!IsExhausted)
            {
                IsActive = true;
                foreach (var each in ConsumedRessources.Items)
                {
                    if (Game.Economy.Inventory[each.Key] < each.Value)
                    {
                        IsActive = false;
                        return;
                    }
                }

                foreach (var each in ConsumedRessources.Items)
                    Game.Economy.Inventory[each.Key] -= each.Value;

                if (Type == StructureType.Mine)
                {
                    Game.Map._Data[x, y].MineralValue -= 1;
                    if (Game.Map._Data[x, y].MineralValue <= 0)
                    {
                        IsExhausted = true;
                        IsActive = false;
                        spriteRenderer.sprite = Game.Config.ExhaustedStructure;
                    }
                }
            }
        }

        public int x
        {
            get
            {
                return (int)transform.position.x;
            }

        }

        public int y
        {
            get
            {
                return (int)transform.position.y;
            }

        }

        void OnDestroy()
        {
            Debug.Log("Structure Destroyed");
            Game.Clock.XCancelInvoke(this);
            if (OnRemove != null)
                OnRemove(this, EventArgs.Empty);
        }

        public void Collect()
        {
            Instantiate(Game.Config.PowerUpFX, gameObject.transform.position + new Vector3(0, 0, -5f), Quaternion.identity);
            if (OnCollect != null)
                OnCollect(this, EventArgs.Empty);
            var report = "Report:\nWe have explored the " + Name + " and found the following things we considered useful:\n";
            foreach (var each in ProducedRessources.Items)
            {
                var loot = RandFuncs.RndRange(0, each.Value);
                Game.Economy.Inventory[each.Key] += loot;
                report += loot + "x " + each.Key + "\n";
            }
            Game.Map.RemoveStructure(x, y);
            Game.TitleScreen.ShowMessage(report);
        }
    }


    
}