﻿using UnityEngine;
using System.Collections;
using System;

namespace Ld31
{
    public class TitleScreen : Root
    {
        public static event EventHandler OnText;

        public Texture2D TitleGraphic;
        string[] introTexts = new []
        {
            "LOG ENTRY:\nWe have lived in the shelter state for countless generations -- for reasons we only know from ancient legend. Some tell of a great angst that took us and drove us, or as some say, called us underground. We have been content and society adapted. Yet unrest stirs in some. And most in me.\n\n" +
            "For many generations I will be the first to leave this underworld and venture forth, onto the surface, to reclaim what once was our home. I hope i will only be the first...\n\n" +
            "To that end I will begin by setting up a basic settlement to invite many more outside as it grows.\n\n"
                ,
                "GETTING STARTED:\n1.\t\t\t Place a campsite near some <Wood>,\n\t\t\t <Dirt> and possibly a <Ruin> or <Cache>\n\t\t\t for an early boost.\n2.\t\t Add living quarters like a <Hut>.\n3.\t\t and food supply like <Farm>.\n4.\t\t Add other buildings.\n5.\t\t Aim for balanced supply and demand. But\n\t\t\t be careful, ressources are limited."
        };

        public string HelpText = "GETTING STARTED:\n1.\t\t\t Place a campsite near some <Wood>,\n\t\t\t <Dirt> and possibly a <Ruin> or <Cache>\n\t\t\t for an early boost.\n2.\t\t Add living quarters like a <Hut>.\n3.\t\t and food supply like <Farm>.\n4.\t\t Add other buildings.\n5.\t\t Aim for balanced supply and demand. But\n\t\t\t be careful, ressources are limited.";

        string shownString;
        public float speed = 2;
        private int currentPage = 0;
        private int lastClickFrame;
        string message = "";

        public void ShowMessage(string txt)
        {
            message = txt;
            lastClickFrame = Time.frameCount;
        }

        // Use this for initialization
        void Start()
        {

        }
	
        // Update is called once per frame
        void Update()
        {
            if (currentPage < introTexts.Length)
            {
                shownString = introTexts[currentPage].Substring(0, (int)Mathf.Clamp((Time.frameCount - lastClickFrame) / speed, 0, introTexts[currentPage].Length));
                if (Input.GetKeyUp(KeyCode.Space) || Input.GetKeyUp(KeyCode.Return) || Input.GetMouseButtonUp(0) || Input.GetMouseButtonUp(1))
                {
                    if (shownString.Length < introTexts[currentPage].Length)
                    {
                        lastClickFrame = -1000;
                    }
                    else
                    {
                        ++currentPage;
                        lastClickFrame = Time.frameCount;
                    }
                }
            }

            if (message.Length > 0)
            {
                if (Input.GetKeyUp(KeyCode.Space) || Input.GetKeyUp(KeyCode.Return) || Input.GetMouseButtonUp(0) || Input.GetMouseButtonUp(1))
                {
                    if (shownString.Length < message.Length)
                    {
                        lastClickFrame = -1000;
                    }
                }
                shownString = message.Substring(0, (int)Mathf.Clamp((Time.frameCount - lastClickFrame) / speed, 0, message.Length));
            }

            if (Game.Economy.Population <= 0)
            {
                message = "GAME OVER\n\nEveryone is gone. Yet " + Game.Economy.PopulationMax + " fearless souls once lived outside. Maybe soon someone will try again.";
            }

        }

        void OnGUI()
        {
            GUI.skin = Game.Config.Skin;
            GUI.Label(new Rect(Screen.width / 2 - TitleGraphic.width / 2, 32 - TitleGraphic.height / 2, TitleGraphic.width + 4, TitleGraphic.height + 4), TitleGraphic);
            if (currentPage < introTexts.Length)
            {
                if (shownString.Length < introTexts[currentPage].Length && Time.frameCount % 4 == 0)
                {
                    if (OnText != null)
                        OnText(this, EventArgs.Empty);
                }
                GUI.Label(new Rect(Screen.width / 2 - 250, Screen.height / 2 - 250, 500, 500), shownString);
            }

            if (message.Length > 0)
            {
                if (shownString.Length < message.Length && Time.frameCount % 4 == 0)
                {
                    if (OnText != null)
                        OnText(this, EventArgs.Empty);
                }
                GUI.Label(new Rect(Screen.width / 2 - 250, Screen.height / 2 - 250, 500, 500), shownString);

                if (GUI.Button(new Rect(Screen.width / 2 - 250, Screen.height / 2 - 250 - 38, 100, 32), "Close"))
                {
                    if (OnText != null)
                        OnText(this, EventArgs.Empty);
                    message = "";
                }
            }
        }
    }
}