﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace Ld31
{
    public class Toolbar : Root
    {
        public static event EventHandler OnSelect;

        public int IconResolution = 34;
        public int IconPadding = 4;
        public Tool[] Tools;
        private List<Tool> tools = new List<Tool>();
        public bool HasSupplies = false;
        Tool previousSelected = null;
        Rect bounds;

        protected override void Awake()
        {
            base.Awake();
            tools = new List<Tool>(Tools);
        }

        void Start()
        {
            Deselect(this, EventArgs.Empty);
        }

        public Tool Selected
        {
            get
            {
                if (tools.Count == 0)
                    return null;
                //return Tools.Find(tool => tool.IsSelected);
                return tools.Find(tool => tool.IsSelected);
            }
        }

        public int Y
        {
            get
            {
                return (int)(
                    Screen.height - IconResolution * 2
                );
            }
        }

        public int X
        {
            get
            {
                return (int)(
                    Screen.width / 2 - (IconResolution + IconPadding) * tools.Count / 2
                );
            }
        }

        void OnEnable()
        {
            Controls.OnCancel += Deselect;
        }

        void OnDisable()
        {
            Controls.OnCancel -= Deselect;
        }

        void Update()
        {
            if (Game.Ui.Toolbar.Selected != null && Game.Ui.Toolbar.Selected.Structure != null)
                HasSupplies = HasBuildingSupplies(Selected);
            else
                HasSupplies = false;

            if (Selected != previousSelected && OnSelect != null)
                OnSelect(this, EventArgs.Empty);
            previousSelected = Selected;

        }

        public bool HasBuildingSupplies(Tool tool)
        {
            if (tool.Structure == null)
                return false;
            foreach (var each in tool.Structure.BuildRessources.Items)
            {
                if (Game.Economy.Inventory[each.Key] < each.Value)
                    return false;
            }
            return true;
        }

        public bool IsBlocked()
        {
            var x = Game.Ui.Zeiger.x;
            var y = Game.Ui.Zeiger.y;
            int i = 0;

            if (!Game.Map.IsOnMap(x - 1, y - 1) || !Game.Map.IsOnMap(x + 1, y + 1))
                return true;

            if (Game.Ui.firstBuildingPlaced && !Game.Map._Data[x, y].InRange)
                return true;

            if (!CanBuildHere())
                return true;

            if (!HasBuildingSupplies(Selected))
                return true;

            if (Game.Map.IsColliding(x - 1, y - 1))
                i++;
            if (Game.Map.IsColliding(x, y - 1))
                i++;
            if (Game.Map.IsColliding(x + 1, y - 1))
                i++;
            if (Game.Map.IsColliding(x - 1, y))
                i++;
            if (Game.Map.IsColliding(x + 1, y))
                i++;
            if (Game.Map.IsColliding(x - 1, y - 1))
                i++;
            if (Game.Map.IsColliding(x, y - 1))
                i++;
            if (Game.Map.IsColliding(x + 1, y - 1))
                i++;
            if (i > 5)
                return true;

            if (
                Game.Map.IsColliding(x - 1, y - 1) &&
                Game.Map.IsColliding(x, y - 1) &&
                Game.Map.IsColliding(x, y - 2) &&
                Game.Map.IsColliding(x + 1, y - 1))
            {
                return true;
            }
            if (
                Game.Map.IsColliding(x - 1, y + 1) &&
                Game.Map.IsColliding(x, y + 1) &&
                Game.Map.IsColliding(x, y + 2) &&
                Game.Map.IsColliding(x + 1, y + 1))
            {
                return true;
            }

            if (
                Game.Map.IsColliding(x - 2, y) &&
                Game.Map.IsColliding(x - 1, y) &&
                Game.Map.IsColliding(x - 1, y - 1) &&
                Game.Map.IsColliding(x - 1, y + 1))
            {
                return true;
            }
            if (
                Game.Map.IsColliding(x + 2, y) &&
                Game.Map.IsColliding(x + 1, y) &&
                Game.Map.IsColliding(x + 1, y - 1) &&
                Game.Map.IsColliding(x + 1, y + 1))
            {
                return true;
            }
            return false;
        }

        public bool CanBuildHere()
        {
            if (!Game.Map.IsOnMap(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
                return false;
            if (Game.Ui.Toolbar.Selected.Structure.SoilType == SoilType.Any)
            {
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Dirt)
                    return false;
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Iron)
                    return false;
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Wood)
                    return false;
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Grass)
                    return false;
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Water)
                    return false;
                if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)SoilType.Stone)
                    return false;
                return true;
            }
            if (Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil == (int)Game.Ui.Toolbar.Selected.Structure.SoilType)
                return true;
            return false;
        }

        // Use this for initialization
        void OnGUI()
        {
            GUI.skin = Game.Config.Skin;
            //GUI.Box(bounds, "");
            DrawGameMenu();
            DrawToolbar();
            DrawTooltip();
        }

        public bool IsMouseOnGUI()
        {
            bounds = new Rect(X - 266, Y - 8, (IconResolution + IconPadding) * Tools.Length + 270, IconResolution + 16);
            Vector3 mouse = new Vector3(Input.mousePosition.x, Screen.height - Input.mousePosition.y, 0);

            return bounds.Contains(mouse);
        }

        private void Deselect(object caller, EventArgs e)
        {
            if (Selected != null)
                Selected.IsSelected = false;
        }

        private void DrawGameMenu()
        {
            if (GUI.Button(new Rect(X - 160 - 2, Y, 128, IconResolution), "Restart"))
                Application.LoadLevel(Application.loadedLevel);
            if (GUI.Button(new Rect(X - 254 - 4, Y, 96, IconResolution), "Quit"))
                Application.Quit();
            if (GUI.Button(new Rect(X - 350 - 4, Y, 96, IconResolution), "Help"))
            {
                Deselect(this, EventArgs.Empty);
                Game.TitleScreen.ShowMessage(Game.TitleScreen.HelpText);
            }
        }

        private void DrawToolbar()
        {
            // draw the toolbar
            int slot = 0;
            foreach (Tool tool in tools)
            {
                if (Game.Ui.firstBuildingPlaced)
                {
                    if (tool.Type == ToolType.REMOVAL || HasBuildingSupplies(tool))
                    {
                        GUI.color = new Color32(255, 255, 255, 255);
                    }
                    else
                    {
                        GUI.color = new Color32(50, 50, 50, 255);
                    }
                    tool.IsSelected = GUI.Toggle(
                        new Rect(
                            X + slot * IconResolution + slot * IconPadding,
                            Y,
                            IconResolution,
                            IconResolution
                        ),
                        tool.IsSelected,
                        new GUIContent("", tool.toolbarIconTexture, tool.Name)

                    );
                }
                else
                {
                    if (tool.IsFirst)
                    {
                        GUI.color = new Color32(255, 255, 255, 255);
                        tool.IsSelected = GUI.Toggle(
                            new Rect(
                                X + slot * IconResolution + slot * IconPadding,
                                Y,
                                IconResolution,
                                IconResolution
                            ),
                            tool.IsSelected,
                            new GUIContent("", tool.toolbarIconTexture, tool.Name)

                        );
                    }
                    else
                    {
                        GUI.color = new Color32(50, 50, 50, 255);
                        GUI.Label(
                            new Rect(
                                X + slot * IconResolution + slot * IconPadding,
                                Y,
                                IconResolution,
                                IconResolution
                            ),
                            new GUIContent("", tool.toolbarIconTexture, tool.Name)

                        );
                    }
                }

                // check if the current toggle is on or off
                if (tool.IsSelected)
                {
                    // set all other toolbar toggles to false
                    foreach (Tool otherTool in tools)
                    {
                        if (otherTool == tool)
                            continue;
                        otherTool.IsSelected = false;
                    }
                }
                slot++;
                GUI.color = new Color32(255, 255, 255, 255);
            }
        }

        private void DrawTooltip()
        {
            if (IsMouseOnGUI())
            {// show the tooltip
                GUI.Label(
                    new Rect(
                        X,
                        Y - IconResolution,
                        500,
                        32
                    ),
                    "> " + GetTooltip()
                );
            }

            GUI.skin = Game.Config.SkinSmall;

            GUI.TextArea(
                new Rect(
                    X,
                    Y - IconResolution * 3,
                    500,
                    64
                ),
                "" + GetNote()
            );
        }

        private string GetNote()
        {
            if (Selected != null)
            {
                return Selected.Note;
            }
            return "";
        }

        string GetSpecialInfo()
        {
            var str = "";
            var block = Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y];

            if (block.Structure.Type == StructureType.PowerUp)
            {
                if (block.InRange)
                    str += ": Investigate?";
                else
                    str += ": Too far to investigate.";
                return str;
            }

            if (block.Structure.IsActive)
                str += ": <color=#7bd5bf>ACTIVE</color>";
            else
                str += ": <color=#858f93>INACTIVE</color>";


            if (block.Structure.Type == StructureType.Mine)
                str += ", " + block.MineralValue + " units left";
            return str;
        }

        public string GetTooltip()
        {

            // === UI TOOLTIP ================================================

            if (IsMouseOnGUI())
            {
                // TOOL SELECTED
                /*if (Game.Ui.Toolbar.Selected != null)
                {
                    // building-tool selected.
                    if (Game.Ui.Toolbar.Selected.Type == ToolType.BUILDING)
                        return "Select <color=cyan>" + Game.Ui.Toolbar.Selected.Name + "</color>";
                }*/
                // NOTHING SELECTED
                return "Select <color=#d5b47b>" + GUI.tooltip + "</color>";
            }

            // === WORLD TOOLTIP =============================================

            // Tool selected
            if (Game.Ui.Toolbar.Selected != null && Game.Map.IsOnMap(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
            {
                switch (Game.Ui.Toolbar.Selected.Type)
                {
                    case ToolType.BUILDING:
                        if (Game.Ui.firstBuildingPlaced && !Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].InRange)
                            return "Out of Range.";
                        if (Game.Map.IsColliding(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
                            return "Blocked.";
                        if (!HasSupplies)
                            return "Insufficient ressources to build.";
                        if (!CanBuildHere())
                            return "Cannot build here. Need <color=#d5b47b>" + Game.Ui.Toolbar.Selected.Structure.SoilType + "</color>";
                        if (IsBlocked())
                            return "Blocked by other Buildings.";

                        return "Build <color=#d5b47b>" + Game.Ui.Toolbar.Selected.Name + "</color>";
                    case ToolType.REMOVAL:
                        if (!Game.Map.IsColliding(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
                            return "Nothing to Remove at " + Game.Ui.Zeiger.x + ", " + Game.Ui.Zeiger.y + ".";
                        return "Remove <color=#d5b47b>" + Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Structure.Name + "</color>.";
                    default:
                        return "-Unknown ToolType-";
                }
            }

            // NOTHING SELECTED, provide info on whats on the map
            if (Game.Map.IsOnMap(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
            {
                if (Game.Map.IsColliding(Game.Ui.Zeiger.x, Game.Ui.Zeiger.y))
                    return "<color=#d5b47b>" + Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Structure.Name + "</color>" + GetSpecialInfo();
                if (new []{ 1, 2, 3, 8, 9, 10 }.Contains(Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil))
                    return "Ressource: <color=#d5b47b>" + (SoilType)Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil + "</color>.";
                return "Terrain: <color=#858f93>" + (SoilType)Game.Map._Data[Game.Ui.Zeiger.x, Game.Ui.Zeiger.y].Soil + "</color>.";

            }
            return "Out of bounds.";
        }
    }
}
